#include "interval.hpp"

int main()
{
	interval x;
	interval y = 1.;
	interval z(1.);

	x = 1.;
	y = 10.;
	z = x / y;

	std::cout << z << "\n";
	std::cout.precision(17);
	std::cout << z << "\n";

	// copy
	x = interval(1., 2.);
	y = interval(3., 4.);
	// assign
	y.assign(3., 4.);

	// basic four operations
	std::cout << x + y << "\n";
	std::cout << x - y << "\n";
	std::cout << x * y << "\n";
	std::cout << x / y << "\n";

	// operation with constant
	std::cout << x + 1 << "\n";
	std::cout << x + 1. << "\n";

	// compound assignment operator
	z += x;
	z += 1.;

	z = interval(3., 4.);
	// access to endpoints
	std::cout << z.lower() << "\n";
	std::cout << z.upper() << "\n";
	z.lower() = 3.5;
	std::cout << z << "\n";

	// static functions
	// whole and hull
	std::cout << interval::whole() << "\n";
	std::cout << interval::hull(1., 2.) << "\n";
	std::cout << interval::hull(2., 1.) << "\n";
	std::cout << interval::hull(1., z) << "\n";
	std::cout << interval::hull(z, 1.) << "\n";
	std::cout << interval::hull(x, y) << "\n";

	// friend functions
	std::cout << width(z) << "\n";
	std::cout << rad(z) << "\n";
	std::cout << median(z) << "\n";
	std::cout << mid(z) << "\n";
	std::cout << norm(z) << "\n";
	std::cout << mag(z) << "\n";
	std::cout << std::boolalpha;
	std::cout << in(3.9, z) << "\n";
	std::cout << in(4.1, z) << "\n";
	std::cout << subset(x, y) << "\n";
	std::cout << subset(y, z) << "\n";
	std::cout << proper_subset(y, z) << "\n";
	std::cout << overlap(x, y) << "\n";
	std::cout << overlap(y, z) << "\n";
	std::cout << intersect(y, z) << "\n";
	std::cout << abs(interval(2., 3.)) << "\n";
	std::cout << abs(interval(-2., 3.)) << "\n";
	std::cout << abs(interval(-2., -1.)) << "\n";
	std::cout << mig(interval(-2., 1.)) << "\n";
	std::cout << mig(interval(-2., -1.)) << "\n";
	std::cout << mig(interval(2., 3.)) << "\n";

	// comparison operators
	std::cout << (x < y) << "\n";
	std::cout << (x < 1.) << "\n";
	std::cout << (1. < x) << "\n";
	std::cout << (interval(1.) == 1.) << "\n";
	std::cout << (interval(1., 2.) != 3.) << "\n";

	// division_part1, division_part2
	// calculate X / (Y \ 0). the result may be divided into two part.
	bool parted;
	std::cout << division_part1(interval(1., 2.), interval(-3., 4.), parted) << "\n";
	// if the result has division_part2, parted is set to true.
	if (parted) std::cout << division_part2(interval(1., 2.), interval(-3., 4.)) << "\n";

	// integer power
	std::cout << pow(interval(2., 3.), 2) << "\n";
	std::cout << pow(interval(2., 3.), -2) << "\n";
	std::cout << pow(interval(-2., 3.), 2) << "\n";
	std::cout << pow(interval(-2., 3.), 3) << "\n";
	// general power
	std::cout << pow(interval(2., 3.), interval(2., 3)) << "\n";
	// mathematical functions
	std::cout << sqrt(interval(2.5, 3.5)) << "\n";
	std::cout << exp(interval(2.5, 3.5)) << "\n";
	std::cout << exp(interval(-std::numeric_limits<double>::infinity(), 0.)) << "\n";
	std::cout << exp(interval(0., std::numeric_limits<double>::infinity())) << "\n";
	std::cout << expm1(interval(-0.25, 0.25)) << "\n";
	std::cout << log(interval(0.75, 1.25)) << "\n";
	std::cout << log(interval(1., std::numeric_limits<double>::infinity())) << "\n";
	std::cout << log(interval(0., 1.)) << "\n";
	std::cout << log1p(interval(-0.25, 0.25)) << "\n";
	std::cout << sin(interval(-0.25, 0.25)) << "\n";
	std::cout << cos(interval(-0.25, 0.25)) << "\n";
	std::cout << tan(interval(-0.25, 0.25)) << "\n";
	std::cout << atan(interval(-0.25, 0.25)) << "\n";
	std::cout << asin(interval(-0.25, 0.25)) << "\n";
	std::cout << acos(interval(-0.25, 0.25)) << "\n";
	std::cout << atan2(interval(1.), interval(1.)) << "\n";
	std::cout << sinh(interval(-0.25, 0.25)) << "\n";
	std::cout << sinh(interval(-std::numeric_limits<double>::infinity(), -10)) << "\n";
	std::cout << cosh(interval(-0.25, 0.25)) << "\n";
	std::cout << cosh(interval(-std::numeric_limits<double>::infinity(), -10)) << "\n";
	std::cout << tanh(interval(-0.25, 0.25)) << "\n";
	std::cout << asinh(interval(-0.25, 0.25)) << "\n";
	std::cout << acosh(interval(1.5, 2.)) << "\n";
	std::cout << atanh(interval(-0.25, 0.25)) << "\n";

	// string is converted to interval which includes the number represented
	// by the string

	// initialize by string. 
	x = "0.1";
	std::cout << x << "\n";
	// operation with string
	std::cout << "0.1" * x << "\n";
	x += "0.1";
	// comparison with string
	std::cout << ("0.20001" > x) << "\n";

	// numeric constants
	std::cout << interval::pi() << "\n";
	std::cout << interval::e() << "\n";
	std::cout << interval::ln2() << "\n";
}
